;********************************************************************
;             ATA/ATAPI-4 Routines
;
;     - Low level ATA/ATAPI Routines
;     - Low level Ram Routines
;     
;
; Ver 1.0                                       Leandro Gentili /1999   
;********************************************************************

RD_STATUS         
	      movlw D_STATUS      ; Status 
              call RUTR 
              movwf STATUS_R
              return
;*************************
;*************************
RD_STATUS_A     
              movlw D_STATUS_A    ; Alternate Status 
              call RUTR 
              movwf STATUS_A
              return
;*************************
;*************************
RD_SC
              movlw D_SECTOR_C    ; Sector Count
              call RUTR
              movwf SECTOR_C
              return
;*************************
;*************************
RD_SN
	      movlw D_SECTOR_N    ; Sector Number 
              call RUTR                
       	      movwf SECTOR_N 
              return
;*************************
;*************************
RD_CL
              movlw D_CILYNDER_L  ; Cylinder Low
              call RUTR
              movwf CILYNDER_L
              return
;***************************
;***************************
RD_CH
              movlw D_CILYNDER_H  ; Cylinder High   
              call RUTR
              movwf CILYNDER_H   	
              return
;*************************
;*************************
RD_ERRORR
              movlw D_ERRORR      ; Error
              call RUTR
              movwf ERRORR
              return
;*************************
;*************************
RD_DATA_R                         ; Data
              bsf STATUS,RP0      ; PortC, PortD IN 
 	      movlw 0xFF              
              movwf TRISD         
	      movwf TRISC
	      bcf STATUS,RP0        

              movlw D_DATA_R      ; Register ATA 
              movwf PORTB

              movlw READ_IDE       
              movwf PORTE

              nop
              movf PORTC,w        ; Read DD0..DD7 IDE
              movwf DATA_RL       
              movf PORTD,w        ; Read DD8..DD15 IDE
              movwf DATA_RH
              clrf PORTE	  ; RDn high

              return                                  
;*****************************************
;*****************************************
WR_COMMAND          
              movf COMMAND,w      ; Command       
              movwf TEMPW
              movlw D_COMMAND
              call RUTW
              return
;*****************************************
;*****************************************
WR_SC	                 
	      movf SECTOR_C,w     ; Sector Count
              movwf TEMPW
	      movlw D_SECTOR_C     
	      call RUTW
	      return
;*****************************************
;*****************************************
WR_SN
	      movf SECTOR_N,w     ; Sector number
	      movwf TEMPW
	      movlw D_SECTOR_N    
	      call RUTW
	      return
;*****************************************
;*****************************************
WR_CL
	      movf CILYNDER_L,w   ; Cylinder Low
              movwf TEMPW
	      movlw D_CILYNDER_L     
              call RUTW
              return
;*****************************************
;*****************************************
WR_CH
	      movf CILYNDER_H,w   ; Cylinder High
	      movwf TEMPW
      	      movlw D_CILYNDER_H     
	      call RUTW
              return
;*****************************************
;*****************************************
WR_DC
	      movf DEVICE_C,w     ; Device Control
              movwf TEMPW
	      movlw D_DEVICE_C
              call RUTW
              return
;*****************************************
;*****************************************
WR_DH
	      movf DEVICE_H,w     ; Device Head 
	      movwf TEMPW
              movlw D_DEVICE_H
	      call RUTW
              return
;*****************************************
;*****************************************
WR_FEATURES
              movf FEATURES,w     ; Features
	      movwf TEMPW
	      movlw D_FEATURES
	      call RUTW
              return
;*****************************************
;*****************************************
WR_DATA_R                         ; Data
              bsf STATUS,RP0      ; PortC, PortD OUT
              clrf TRISD              
	      clrf TRISC
	      bcf STATUS,RP0       

              movlw D_DATA_R	  ; TEMPWistro ATA
              movwf PORTB

              movf DATA_RL,w      ; Write DD0..DD7 IDE 
              movwf PORTC         
              movf DATA_RH,w      ; Write DD8..DD15 IDE
              movwf PORTD
      
              movlw WRITE_IDE     
              movwf PORTE
              clrf PORTE	
 
              return
;*****************************************
;*****************************************
RUTR          
              movwf PORTB         ; TEMPWistro ATA 

              bsf STATUS,RP0      ; PortC IN 
              movlw 0xFF
              movwf TRISC
              movwf TRISD
              bcf STATUS,RP0
 
              movlw READ_IDE       
              movwf PORTE

              nop
              movf PORTC,w        ; Read DD0..D7 IDE
              clrf PORTE	  ; RDn High                     

	      return
;*****************************************
;*****************************************
RUTW
              movwf PORTB         ; TEMPWistro ATA

              bsf STATUS,RP0      ; PortC OUT
	      clrf TRISC        
              clrf TRISD
	      bcf STATUS,RP0       

              movf TEMPW,w        ; Write DD0..DD7 IDE 
              movwf PORTC
              
              movlw WRITE_IDE  
              movwf PORTE      
              clrf PORTE	
       
              return
;*****************************************
;*****************************************   
RD_RAM  	        
              bsf STATUS,RP0      ; PortC,OUT PortD IN
 	      movlw 0xFF         
	      movwf TRISD        
              clrf TRISC
	      bcf STATUS,RP0
       
              movf ADR_RL,w       ; PortC A0..A7 
              movwf PORTC        
              movf ADR_RH,w       ; PortB A8..A15
              movwf PORTB

              movlw READ_RAM      
	      movwf PORTE
                              
              nop 
	      movf PORTD,w        ; Read Byte from RAM 
              movwf BYTERAM      
              clrf PORTE	  ; RDn High   

              return
;*****************************************
;*****************************************   
WR_RAM 
              bsf STATUS,RP0      ; PortC, PortD OUT       
	      clrf TRISD        
              clrf TRISC
	      bcf STATUS,RP0

              movf ADR_WL,w       ; PortC A0..A7  
              movwf PORTC
              movf ADR_WH,w       ; PortB A8..A15
              movwf PORTB   

              movf BYTERAM,w      ; Write Byte to RAM 
              movwf PORTD         

              movlw WRITE_RAM
              movwf PORTE     
              clrf PORTE	
               
	      return
;**************************************************************************
;**************************************************************************
ST1                               ; Wait: 
              call RD_STATUS      ; Bsy =0 
              movf STATUS_R,W     ; Drq =0
              andlw b'10001000'     
              btfsc STATUS,Z
              return
              btfss STATUS_R,ERR
              goto ST1
              return
;**************************************************************************
;**************************************************************************
ST2                               ; Wait: 
              call RD_STATUS      ; Bsy =0 
              movf STATUS_R,W     ; Drq =1
              andlw b'10001000'     
              sublw b'00001000'
              btfsc STATUS,Z
              return
              btfss STATUS_R,ERR
              goto ST2               
              return                       
;**************************************************************************
;**************************************************************************
RD_BLOCK 
              call RD_ERRORR
              call RD_SC
              call RD_SN
              call RD_CL
              call RD_CH
              return
;**************************************************************************
;**************************************************************************
WR_BLOCK                
              call WR_FEATURES
              call WR_SC
              call WR_SN
              call WR_CL 
              call WR_CH
              call WR_COMMAND
              return
;**************************************************************************
;**************************************************************************
WR_PKT                            ; PACKET PARAMETERS 
              movf BP0,w
              movwf DATA_RL
              movf BP1,w
              movwf DATA_RH
              call WR_DATA_R
              movf BP2,w
              movwf DATA_RL
              movf BP3,w
              movwf DATA_RH
              call WR_DATA_R         
              movf BP4,w
              movwf DATA_RL
              movf BP5,w
              movwf DATA_RH
              call WR_DATA_R
              movf BP6,w
              movwf DATA_RL
              movf BP7,w
              movwf DATA_RH
              call WR_DATA_R
              movf BP8,w
              movwf DATA_RL
              movf BP9,w
              movwf DATA_RH
              call WR_DATA_R
              movf BP10,w
              movwf DATA_RL
              movf BP11,w
              movwf DATA_RH
              call WR_DATA_R
              return          
;**************************************************************************
;**************************************************************************
DEVICE_SELECT            
              call ST1
              clrf DEVICE_H
              call WR_DH
              call ST1
              return
;**************************************************************************
;**************************************************************************
INI_ATAPI     
              call DEVICE_SELECT                                                    
              movlw b'00000010'    
              movwf DEVICE_C
              call WR_DC         
              call ST1
                      
              call RD_BLOCK
              movf CILYNDER_L,w
              sublw 0x14
              btfss STATUS,Z
              goto N_ATAPI

              movf CILYNDER_H,w
              sublw 0xEB
              btfss STATUS,Z
              goto N_ATAPI

              bcf FLAG,ATAPIn
              return                                               
N_ATAPI
              bsf FLAG,ATAPIn
              return
;--------------------------------------------------------------------------
               ; ATA COMMANDs
DEVICE_RESET
              call DEVICE_SELECT
              clrf FEATURES
              clrf SECTOR_C
              clrf SECTOR_N
              clrf CILYNDER_L
              clrf CILYNDER_H
              clrf DEVICE_H
              movlw 0x08
              movwf COMMAND
              call WR_BLOCK
              return
             
WR_C_P                            ; ATAPI Command Packet
              call ST1 
              clrf FEATURES       ; Byte count limit 16k            
              clrf SECTOR_C         
              clrf SECTOR_N	  
              movlw 0xFF
              movwf CILYNDER_L
              movlw 0xFF
              movwf CILYNDER_H
              clrf DEVICE_H
              movlw 0xA0 
              movwf COMMAND                  
              call WR_BLOCK
              return
;---------------------------------------------------------------------------
               ; ATAPI COMMANDs                    
READ10         
              call WR_C_P
              call ST2
              movlw 0x28
              movwf BP0
              clrf BP1
              movf LBA_1,w
              movwf BP5
              movf LBA_2,w
              movwf BP4
              movf LBA_3,w
              movwf BP3
              movf LBA_4,w
              movwf BP2   
              clrf BP6
              movf TRANS_2,w
	      movwf BP7	 			
              movf TRANS_1,w      ; Read 8 blockes (16Kbytes)
              movwf BP8
              clrf BP9
              clrf BP10
              clrf BP11
              call WR_PKT
              return

PLAY_AUDIO_MSF                
              call WR_C_P
              call ST2
              movlw 0x47
              movwf BP0
              clrf BP1
	      clrf BP2
              movf START_M,w
              movwf BP3
              movf START_S,w
              movwf BP4
              movf START_F,w
              movwf BP5
              movf END_M,w
              movwf BP6
              movf END_S,w
              movwf BP7
              movf END_F,w
              movwf BP8
              clrf BP9
              clrf BP10
              clrf BP11
              call WR_PKT
              call RD_STATUS_A
              call ST1
              return

READ_TOC
              call WR_C_P
              call ST2
              movlw 0x43
              movwf BP0
	      movlw 0x02  
              movwf BP1
	      clrf BP2
              clrf BP3
              clrf BP4
              clrf BP5
              clrf BP6
              movlw 0x10
              movwf BP7
              clrf BP8
              clrf BP9
              clrf BP10
              clrf BP11
              call WR_PKT               
              call RD_STATUS_A
              call ST2 
              return

REQUEST_SENSE
              call WR_C_P
              call ST2
              movlw 0x03
              movwf BP0
              clrf BP1
	      clrf BP2
              clrf BP3
              movlw 0x12
              movwf BP4
              clrf BP5
              clrf BP6
              clrf BP7
              clrf BP8
              clrf BP9
              clrf BP10
              clrf BP11
              call WR_PKT               
              call RD_STATUS_A
              call ST2            
              btfsc STATUS_R,ERR
              return               
	                          ; Byte Returned
              call RD_DATA_R      ;0,1
              call RD_DATA_R      ;2,3 
              movf DATA_RL,w        
              andlw b'00001111'     
              movwf TEMP1         ; Sense Key
              call RD_DATA_R      ;4,5
              call RD_DATA_R      ;6,7
              call RD_DATA_R      ;8,9
              call RD_DATA_R      ;10,11
              call RD_DATA_R      ;12,13
              movf DATA_RL,w
              movwf TEMP2         ; ASC
              movf DATA_RH,w
              movwf TEMP3         ; ASCQ
              call RD_DATA_R      ;14,15 
              call RD_DATA_R      ;16,17
              return

ST_UNIT
              call WR_C_P
              call ST2
              movlw 0x1B
              movwf BP0
              clrf BP1
	      clrf BP2
              clrf BP3
              movf FLAG_CD,w
              movwf BP4
              clrf BP5
              clrf BP6
              clrf BP7
              clrf BP8
              clrf BP9
              clrf BP10
              clrf BP11
              call WR_PKT               
              call RD_STATUS_A
              call ST1 
              return

PAUSE_RESUME
              call WR_C_P
              call ST2
              movlw 0x4B
              movwf BP0
              clrf BP1
	      clrf BP2
              clrf BP3
              clrf BP4
              clrf BP5
              clrf BP6
              clrf BP7
              movlw 0x00          
              btfsc FLAG_CD,3
              movlw 0x01         
              movwf BP8               
              clrf BP9
              clrf BP10
              clrf BP11
              call WR_PKT               
              call RD_STATUS_A
              call ST1 
              return

READ_SUB
              call WR_C_P
              call ST2
              movlw 0x42
              movwf BP0
              movlw 0x02
              movwf BP1
              movlw 0x40
              movwf BP2
              movlw 0x01
              movwf BP3
              clrf BP4
              clrf BP5
              clrf BP6
              clrf BP7
              movlw .15
              movwf BP8
              clrf BP9
              clrf BP10
              clrf BP11
              call WR_PKT               
              call RD_STATUS_A
              call ST2 
	                          ; Byte Returned
              call RD_DATA_R      ;1
              movf DATA_RH,w                  
              movwf TEMP1
              call RD_DATA_R       
              call RD_DATA_R      
              call RD_DATA_R
              call RD_DATA_R
              call RD_DATA_R
              call RD_DATA_R
              call RD_DATA_R
              return

SET_MAX_SPEED 
              call WR_C_P
              call ST2
              movlw 0xBB
              movwf BP0
              clrf BP1
	      movlw 0x00          ;Set Max Speed x1 (150 KB/s)  
              movwf BP2                    
              movlw 0x96
              movwf BP3
              clrf BP4
              clrf BP5
              clrf BP6
              clrf BP7
              clrf BP8
              clrf BP9
              clrf BP10
              clrf BP11
              call WR_PKT               
              return
               
;--------------------------------------------------------------------------
